from sanic.views import HTTPMethodView
from sanic.response import json, file
from naas.types import t_static
import uuid

class LogsController(HTTPMethodView):
    __logger = None
    
    def __init__(self, logger, *args, **kwargs):
        super(LogsController, self).__init__(*args, **kwargs)
        self.__logger = logger
    
    async def get(self, request):
        # return json({'data': [], 'totalRecords': 0})
        as_file = request.args.get('as_file', False)
        if as_file:
            return await file(self.__logger.get_file_path(), attachment_filename='logs.csv')
        else:
            uid = str(uuid.uuid4())
            limit = int(request.args.get('limit', 0))
            skip = int(request.args.get('skip', 0))
            search = str(request.args.get('search', ''))
            filters = list(request.args.get('filters', []))
            logs = self.__logger.list(uid, skip, limit, search, filters)
            self.__logger.info(
                {'id': uid, 'type': t_static, 'status': 'send', 'filepath': 'logs', 'skip': skip, 'limit': limit, 'search': search, 'filters': filters})
            return json(logs)


from sanic.views import HTTPMethodView
from sanic import response
from naas.types import t_static, t_notebook, t_health, t_error
import uuid
import json
import os

class StaticController(HTTPMethodView):
    __logger = None
    __jobs = None
    __path_lib_files = None
    __path_html_error = None
    __assets_files = 'assets'
    __html_files = 'html'
    __manager_error = 'error.html'
    
    def __init__(self, logger, jobs, path_assets, *args, **kwargs):
        super(StaticController, self).__init__(*args, **kwargs)
        self.__logger = logger
        self.__jobs = jobs
        self.__path_lib_files = path_assets
        self.__path_html_error = os.path.join(self.__path_lib_files, self.__html_files, self.__manager_error)
        print('self.__path_html_error', self.__path_html_error)

    def __html_error(self, content):
        try:
            with open(self.__path_html_error, 'r') as f:
                template = f.read()
                f.close()
                return template.replace('{ERROR}', content)
        except Exception:
            print('Cannot get html error')
            return ''

    def get(self, request, token):
        path = None 
        if (token == 'up' or token == 'down'):
            return response.file(os.path.join(self.__path_lib_files, self.__assets_files, f"{token}.png"))
        else:
            uid = str(uuid.uuid4())
            task = self.__jobs.find_by_value(uid, token, t_static)
            if task:
                file_filepath = task.get('path')
                file_filename = os.path.basename(file_filepath)
                params = task.get('params', dict())
                self.__logger.info(
                    {'id': uid, 'type': t_static, 'status': t_start, 'filepath': file_filepath, 'token': token})
                try:
                    self.__jobs.update(uid, file_filepath, t_static, token,
                                params, t_health, 1)
                    res = response.file(file_filepath, attachment_filename=file_filename)
                    self.__logger.info(
                        {'id': uid, 'type': t_static, 'status': t_start, 'filepath': file_filepath, 'token': token})
                    return res
                except Exception as e:
                    self.__logger.error(
                        {'id': uid, 'type': t_static, 'status': t_error, 'filepath': file_filepath, 'token': token, "error": e})
                    self.__jobs.update(uid, file_filepath, t_static,
                                token, params, t_error, 1)
                    return response.html(self.__html_error(json.dumps({'id': uid, "error": e})))
            self.__logger.error({'id': uid, 'type': t_static, 'status': t_error,
                                    "error": 'Cannot find your token', 'token': token})
            return response.html(self.__html_error(json.dumps({'id': uid, "error": "Cannot find your token", 'token': token})))





from sanic.views import HTTPMethodView
from sanic.response import json
from naas.types import t_health, t_static
import uuid

class EnvController(HTTPMethodView):
    __notif_url = None
    __user = None
    __logger = None
    
    def __init__(self, logger, user, public_url, proxy_url,, notif_url tz, *args, **kwargs):
        super(EnvController, self).__init__(*args, **kwargs)
        self.__logger = logger
        self.__notif_url = notif_url
        self.__user = user
        self.__public_url = public_url
        self.__proxy_url = proxy_url
        self.__tz = tz

    async def get(self, request):
        uid = str(uuid.uuid4())
        env = {
            'status': t_health,
            'notif_url': self.__notif_url,
            'JUPYTERHUB_USER': self.__user,
            'JUPYTERHUB_URL': self.__public_url,
            'PUBLIC_PROXY_API': self.__proxy_url,
            'TZ': self.__tz
            
        }
        self.__logger.info(
            {'id': uid, 'type': t_static, 'status': 'send', 'filepath': 'env'})
        return json(env)


from sanic.views import HTTPMethodView
from sanic.response import json
from naas.types import t_static, t_notebook, t_health, t_error
import uuid

class SchedulerController(HTTPMethodView):
    __logger = None
    __scheduler = None
    
    def __init__(self, logger, scheduler, *args, **kwargs):
        super(SchedulerController, self).__init__(*args, **kwargs)
        self.__logger = logger
        self.__scheduler = scheduler
    
    async def get(self, request, mode):
        if (mode == 'pause'):
            self.__scheduler.pause()
        elif (mode == 'resume'):
            self.__scheduler.resume()
        return json({"status": self.__scheduler.status()})


from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
import os

class FixUvloop():
    def __init__(self):
        try :
            import uvloop
            with open(uvloop.__file__, 'w') as fp:
                fp.write("raise ImportError\n")
                pass
        except ImportError:
            pass
        
class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        develop.run(self)
        FixUvloop()
        
class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        FixUvloop()

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='naas',
    version='0.0.16',
    scripts=['scripts/naas'],
    author="Martin Donadieu",
    author_email="martindonadieu@gmail.com",
    description="scheduler system for notebooks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cashstory/naas",
    packages=find_packages(exclude=["tests"]),
    package_data={
        "naas": ["runner/html/*.html", "runner/assets/*.png"],
    },
    cmdclass={
        'develop': PostDevelopCommand,
        'install': PostInstallCommand,
    },
    install_requires=[
        "papermill>=2,<3",
        "pretty-cron>=1,<2",
        "APScheduler>=3,<4",
        "pycron>=3,<4",
        "pandas>=1,<2",
        "daemonize>=2,<3",
        "escapism>=1,<2",
        "notebook>=6,<7",
        "ipython>=7,<8",
        "ipykernel>=5,<6",
        "requests>=2,<3",
        "sentry-sdk>=0,<1",
        "sanic>=20,<21",
        "sanic-openapi>=0,<1",
        "argparse>=1,<2",
        "nbconvert>=6,<7",
        "nbclient>=0,<1",
        "beautifulsoup4>=4,<5",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from sanic.views import HTTPMethodView
from sanic import response
from naas.types import t_static, t_task, t_error
import uuid
import json

class JobsController(HTTPMethodView):
    __jobs = None
    __logger = None
    
    def __init__(self, logger, jobs, *args, **kwargs):
        super(JobsController, self).__init__(*args, **kwargs)
        self.__jobs = jobs
        self.__logger = logger

    async def get(self, request):
        uid = str(uuid.uuid4())
        status = self.__jobs.list(uid)
        self.__logger.info(
            {'id': uid, 'type': t_static, 'status': 'send', 'filepath': 'status'})
        return response.json(status)
    
    async def post(self, request):
        uid = str(uuid.uuid4())
        data = request.json
        if not data or ["path", "type", "params", "value", "status"].sort() != list(data.keys()).sort():
            self.__logger.info(
                {'id': uid, 'type': t_task, 'status': t_error, 'error': 'missing keys', 'tb': data})
            return response.json({'result': 'missing keys'}, status=400)
        updated = await self.__jobs.update(uid, 
            data['path'], data['type'], data['value'], data['params'], data['status'])
        self.__logger.info(
            {'id': uid, 'type': t_task, 'status': updated['status']})
        return response.json(updated)


from naas.types import t_static, t_notebook, t_health, t_error, t_start
from sanic.views import HTTPMethodView
from sanic.response import json, file
import asyncio
import uuid

class NbController(HTTPMethodView):
    __logger = None
    
    def __init__(self, logger, jobs, nb, *args, **kwargs):
        super(NbController, self).__init__(*args, **kwargs)
        self.__logger = logger
        self.__jobs = jobs
        self.__nb = nb

    async def get(self, request, token):
        uid = str(uuid.uuid4())
        task = self.__jobs.find_by_value(uid, token, t_notebook)
        if task:
            value = task.get('value', None)
            file_filepath = task.get('path')
            task['params'] = {**(task.get('params', dict())), **(request.args)}
            self.__logger.info(
                {'id': uid, 'type': t_notebook, 'status': t_start, 'filepath': file_filepath, 'token': token})
            res = await self.__nb.exec(uid, task)
            if (res.get('error')):
                self.__logger.error({'main_id': uid, 'id': uid, 'type': t_notebook, 'status': t_error,
                                        'filepath': file_filepath, 'duration': res.get('duration'), 'error': res.get('error')})
                await self.__jobs.update(uid, file_filepath, t_notebook, value, task.get(
                    'params'), t_error, res.get('duration'))
                return json({'id': uid, "error": res.get('error')})
            self.__logger.info(
                {'main_id': uid, 'id': uid, 'type': t_notebook, 'status': t_health, 'filepath': file_filepath, 'duration': res.get('duration')})
            await self.__jobs.update(uid, file_filepath, t_notebook, value, task.get(
                'params'), t_health, res.get('duration'))
            return self.__nb.response(uid, res, res.get('duration'), task.get('params'))
        self.__logger.error({'id': uid, 'type': t_notebook, 'status': t_error,
                                'token': token, 'error': 'Cannot find your token'})
        return json({'id': uid, "error": "Cannot find your token"}, status=404)
